@@ -19,6 +19,7 @@
 #include "charConv.h"
 
 #include "mpc.h"
+#include "gcc.h"
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -44,20 +45,28 @@ static int ignore_invalid;
 
 #define BUFFER_SIZE	1024
 #ifdef HAVE_ICONV
-static void closeCharSetConversion();
+static void closeCharSetConversion(void);
 #endif
 
 /* code from iconv_prog.c (omiting invalid symbols): */
 #ifdef HAVE_ICONV
-static char * skip_invalid(char *to)
+static inline char * mpc_strchrnul(const char *s, int c)
 {
-	const char *errhand = strchrnul(to, '/');
+	char *ret = strchr(s, c);
+	if (!ret)
+		ret = strchr(s, '\0');
+	return ret;
+}
+
+static char * skip_invalid(const char *to)
+{
+	const char *errhand = mpc_strchrnul(to, '/');
 	int nslash = 2;
 	char *newp, *cp;
 
 	if (*errhand == '/') {
 		--nslash;
-		errhand = strchrnul (errhand, '/');
+		errhand = mpc_strchrnul(errhand, '/');
 
 		if (*errhand == '/') {
 			--nslash;
@@ -66,7 +75,8 @@ static char * skip_invalid(char *to)
 	}
 
 	newp = (char *)malloc(errhand - to + nslash + 7 + 1);
-	cp = mempcpy(newp, to, errhand - to);
+	memcpy(newp, to, errhand - to);
+	cp = newp + (errhand - to);
 	while (nslash-- > 0)
 		*cp++ = '/';
 	if (cp[-1] != '/')
@@ -76,10 +86,14 @@ static char * skip_invalid(char *to)
 }
 #endif
 
-static int setCharSetConversion(char * to, char * from) {
+static int
+setCharSetConversion(mpd_unused const char *to, mpd_unused const char *from)
+{
 #ifdef HAVE_ICONV
+	char *allocated = NULL;
+
 	if (ignore_invalid)
-		to = skip_invalid(to);
+		to = allocated = skip_invalid(to);
 	if(char_conv_to && strcmp(to,char_conv_to)==0 &&
 			char_conv_from && strcmp(from,char_conv_from)==0)
 		return 0;
@@ -92,15 +106,33 @@ static int setCharSetConversion(char * to, char * from) {
 	char_conv_to = strdup(to);
 	char_conv_from = strdup(from);
 
-	if (ignore_invalid)
-		free(to);
+	if (allocated != NULL)
+		free(allocated);
 
 	return 0;
 #endif
 	return -1;
 }
 
-static char * convStrDup(char * string) {
+#ifdef HAVE_ICONV
+static inline size_t deconst_iconv(iconv_t cd,
+				   const char **inbuf, size_t *inbytesleft,
+				   char **outbuf, size_t *outbytesleft)
+{
+	union {
+		const char **a;
+		char **b;
+	} deconst;
+
+	deconst.a = inbuf;
+
+	return iconv(cd, deconst.b, inbytesleft, outbuf, outbytesleft);
+}
+#endif
+
+static char *
+convStrDup(mpd_unused const char *string)
+{
 #ifdef HAVE_ICONV
 	char buffer[BUFFER_SIZE];
 	size_t inleft = strlen(string);
@@ -117,9 +149,10 @@ static char * convStrDup(char * string) {
 	while(inleft) {
 		bufferPtr = buffer;
 		outleft = BUFFER_SIZE;
-		err = iconv(char_conv_iconv,&string,&inleft,&bufferPtr,
-					&outleft);
-		if(outleft==BUFFER_SIZE || (err<0 && errno!=E2BIG)) {
+		err = deconst_iconv(char_conv_iconv,&string,&inleft,&bufferPtr,
+				    &outleft);
+		if (outleft == BUFFER_SIZE ||
+		    (err == (size_t)-1 && errno != E2BIG)) {
 			free(ret);
 			return NULL;
 		}
@@ -163,7 +196,7 @@ void setLocaleCharset(void) {
                 if((temp = nl_langinfo(CODESET))) {
                         charset = strdup(temp);
                 }
-                if(!setlocale(LC_CTYPE,originalLocale));
+		setlocale(LC_CTYPE,originalLocale);
         }
 	
 	if(localeCharset) free(localeCharset);
@@ -178,7 +211,7 @@ void setLocaleCharset(void) {
         localeCharset = strdup("ISO-8859-1");
 }
 
-char * toUtf8(char * from) {
+char * toUtf8(const char * from) {
 	static char * to = NULL;
 
 	if(to) free(to);
@@ -191,7 +224,7 @@ char * toUtf8(char * from) {
 	return to;
 }
 
-char * fromUtf8(char * from) {
+char * fromUtf8(const char * from) {
 	static char * to = NULL;
 
 	if(to) free(to);
