@@ -25,6 +25,7 @@
 #include "status.h"
 #include "command.h"
 #include "mpc.h"
+#include "gcc.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -33,17 +34,18 @@
 #include <time.h>
 #include <sys/param.h>
 
-#define DIE(args...) do { fprintf(stderr, ##args); return -1; } while(0)
+#define DIE(...) do { fprintf(stderr, __VA_ARGS__); return -1; } while(0)
 
 #define SIMPLE_CMD(funcname, libmpdclient_funcname, ret) \
-int funcname ( int argc, char ** argv, mpd_Connection * conn) { \
+int funcname(mpd_unused int argc, mpd_unused char **argv, \
+	     mpd_Connection *conn) { \
         libmpdclient_funcname(conn); \
         my_finishCommand(conn); \
         return ret; \
 }
 
 #define SIMPLE_ONEARG_CMD(funcname, libmpdclient_funcname, ret) \
-int funcname ( int argc, char ** argv, mpd_Connection * conn) { \
+int funcname (mpd_unused int argc, char **argv, mpd_Connection *conn) { \
         libmpdclient_funcname(conn, toUtf8(argv[0])); \
         my_finishCommand(conn); \
         return ret; \
@@ -79,7 +81,7 @@ static mpd_Status * getStatus(mpd_Connection * conn) {
 	return ret;
 }
 
-int isUrl(char * s) {
+static int isUrl(char * s) {
 	char * t;
 
 	t = strstr(s,"://");
@@ -98,9 +100,9 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 	List ** lists;
 	mpd_InfoEntity * entity;
 	ListNode * node;
-	mpd_Song * song;
+	struct mpd_song *song;
 	char * sp;
-	char * dup;
+	char * duplicated;
 	int i;
 	int * arglens;
 	int len;
@@ -124,9 +126,9 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 	while((entity = mpd_getNextInfoEntity(conn))) {
 		if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
 			song = entity->info.song;
-			sp = dup = strdup(fromUtf8(song->file));
+			sp = duplicated = strdup(fromUtf8(song->file));
 			while((sp = strchr(sp,' '))) *sp = '_';
-			len = strlen(dup);
+			len = strlen(duplicated);
 			for(i=0;i<argc;i++) {
 				if (strcmp(argv[i], "/") == 0) {
 					insertInListWithoutKey(lists[i],
@@ -134,11 +136,11 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 					continue;
 				}
 				if(len<arglens[i]) continue;
-				ret = strncmp(argv[i],dup,
+				ret = strncmp(argv[i], duplicated,
 						arglens[i]);
 				if(ret==0 && (argv[i][arglens[i]-1] == '/' ||
-						dup[arglens[i]] == '\0' || 
-						dup[arglens[i]] == '/')) 
+					      duplicated[arglens[i]] == '\0' ||
+					      duplicated[arglens[i]] == '/'))
 				{
 					insertInListWithoutKey(
 							lists[i],
@@ -146,7 +148,7 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 									song->file)));
 				}
 			}
-			free(dup);
+			free(duplicated);
 		}
 		mpd_freeInfoEntity(entity);
 	}
@@ -181,7 +183,8 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-int cmd_crop( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_crop(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	mpd_Status *status = getStatus( conn );
 	int length = ( status->playlistLength - 1 );
@@ -261,8 +264,15 @@ int cmd_del ( int argc, char ** argv, mpd_Connection * conn )
 		else
 			DIE("error parsing song numbers from: %s\n",argv[i]);
 
-		if(range[0]<=0 || range[1]<=0 || range[1]<range[0])
-			DIE("song numbers must be positive: %i-%i\n",range[0],range[1]);
+		if(range[0]<=0 || range[1]<=0) {
+			if (range[0]==range[1])
+				DIE("song number must be positive: %i\n",range[0]);
+			else
+				DIE("song numbers must be positive: %i to %i\n",range[0],range[1]);
+		}
+
+		if(range[1]<range[0])
+			DIE("song range must be from low to high: %i to %i\n",range[0],range[1]);
 
 		if(range[1]>plLength)
 			DIE("song number does not exist: %i\n",range[1]);
@@ -287,7 +297,8 @@ int cmd_del ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-int cmd_toggle( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_toggle(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	mpd_Status * status;
 	status = getStatus(conn);
@@ -300,7 +311,8 @@ int cmd_toggle( int argc, char ** argv, mpd_Connection * conn )
 	return 1;
 }
 
-int cmd_outputs( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_outputs(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	mpd_OutputEntity * output;
 
@@ -328,7 +340,8 @@ int cmd_outputs( int argc, char ** argv, mpd_Connection * conn )
 	return( 0 );
 }
 
-int cmd_enable( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_enable(mpd_unused int argc, char **argv, mpd_Connection *conn)
 {
 	int arg;
 
@@ -339,11 +352,11 @@ int cmd_enable( int argc, char ** argv, mpd_Connection * conn )
 	}
 
 	mpd_finishCommand( conn );
-	cmd_outputs( 0, NULL, conn );
-	return( 0 );
+	return cmd_outputs(0, NULL, conn);
 }
 
-int cmd_disable( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_disable(mpd_unused int argc, char **argv, mpd_Connection *conn)
 {
 	int arg;
 
@@ -352,10 +365,9 @@ int cmd_disable( int argc, char ** argv, mpd_Connection * conn )
 	} else {
 		mpd_sendDisableOutputCommand( conn, ( arg - 1 ) );
 	}
-	
+
 	mpd_finishCommand( conn );
-	cmd_outputs( 0, NULL, conn );
-	return( 0 );
+	return cmd_outputs(0, NULL, conn);
 }
 
 int cmd_play ( int argc, char ** argv, mpd_Connection * conn )
@@ -365,6 +377,8 @@ int cmd_play ( int argc, char ** argv, mpd_Connection * conn )
 
 	if(0==argc) song = MPD_PLAY_AT_BEGINNING;
 	else {
+		mpd_Status *status;
+
 		for(i=0;i<argc-1;i++)
 			printf("skipping: %s\n",argv[i]);
 
@@ -374,7 +388,7 @@ int cmd_play ( int argc, char ** argv, mpd_Connection * conn )
 		song--;
 
 		/* This is necessary, otherwise mpc will output the wrong playlist number */
-		mpd_Status * status = getStatus(conn);
+		status = getStatus(conn);
 		i = status->playlistLength;
 		mpd_freeStatus(status);
 		if(song >= i)
@@ -387,7 +401,8 @@ int cmd_play ( int argc, char ** argv, mpd_Connection * conn )
 	return 1;
 }
 
-int cmd_seek ( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_seek(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	mpd_Status * status;
 	char * arg = argv[0];
@@ -519,7 +534,8 @@ int cmd_seek ( int argc, char ** argv, mpd_Connection * conn )
 	return 1;
 }
 
-int cmd_move ( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_move(mpd_unused int argc, char **argv, mpd_Connection *conn)
 {
 	int from;
 	int to;
@@ -540,7 +556,8 @@ int cmd_move ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-int cmd_playlist ( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_playlist(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	mpd_InfoEntity * entity;
 	mpd_Status * status;
@@ -556,7 +573,7 @@ int cmd_playlist ( int argc, char ** argv, mpd_Connection * conn )
 
 	while((entity = mpd_getNextInfoEntity(conn))) {
 		if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
-			mpd_Song * song = entity->info.song;
+			struct mpd_song *song = entity->info.song;
 
 			printf("%s%i) ", (status->song == count)?">":" ", 1+count);
 			pretty_print_song(song);
@@ -575,8 +592,7 @@ int cmd_playlist ( int argc, char ** argv, mpd_Connection * conn )
 
 int cmd_listall ( int argc, char ** argv, mpd_Connection * conn )
 {
-	mpd_InfoEntity * entity;
-	char * listall = "";
+	const char * listall = "";
 	int i=0;
 
 	if(argc>0) listall = toUtf8(argv[i]);
@@ -585,13 +601,7 @@ int cmd_listall ( int argc, char ** argv, mpd_Connection * conn )
 		mpd_sendListallCommand(conn,listall);
 		printErrorAndExit(conn);
 
-		while((entity = mpd_getNextInfoEntity(conn))) {
-			if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
-				mpd_Song * song = entity->info.song;
-				printf("%s\n",fromUtf8(song->file));
-			}
-			mpd_freeInfoEntity(entity);
-		}
+		print_filenames(conn);
 
 		my_finishCommand(conn);
 
@@ -602,7 +612,7 @@ int cmd_listall ( int argc, char ** argv, mpd_Connection * conn )
 
 int cmd_update ( int argc, char ** argv, mpd_Connection * conn) 
 {
-	char * update = "";
+	const char * update = "";
 	int i = 0;
 
 	mpd_sendCommandListBegin(conn);
@@ -625,12 +635,15 @@ int cmd_update ( int argc, char ** argv, mpd_Connection * conn)
 int cmd_ls ( int argc, char ** argv, mpd_Connection * conn )
 {
 	mpd_InfoEntity * entity;
-	char * ls = "";
+	char * ls;
 	int i = 0;
 	char * sp;
 	char * dp;
 
-	if(argc>0) ls = toUtf8(argv[i]);
+	if (argc > 0)
+		ls = toUtf8(argv[i]);
+	else
+		ls = strdup("");
 
 	do {
 		mpd_sendListallCommand(conn,"");
@@ -664,7 +677,7 @@ int cmd_ls ( int argc, char ** argv, mpd_Connection * conn )
 				printf("%s\n",fromUtf8(dir->path));
 			}
 			else if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
-				mpd_Song * song = entity->info.song;
+				struct mpd_song *song = entity->info.song;
 				printf("%s\n",fromUtf8(song->file));
 			}
 			mpd_freeInfoEntity(entity);
@@ -680,7 +693,7 @@ int cmd_ls ( int argc, char ** argv, mpd_Connection * conn )
 int cmd_lsplaylists ( int argc, char ** argv, mpd_Connection * conn )
 {
 	mpd_InfoEntity * entity;
-	char * ls = "";
+	const char * ls = "";
 	int i = 0;
 
 	if(argc>0) ls = toUtf8(argv[i]);
@@ -747,35 +760,20 @@ int cmd_load ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-int cmd_search ( int argc, char ** argv, mpd_Connection * conn ) 
+static int do_search ( int argc, char ** argv, mpd_Connection * conn, int exact ) 
 {
-	mpd_InfoEntity * entity;
-	int type, i, numconstraints = 0;
-	struct _constraints {
-		int type;
-		char * query;
-	} * constraints;
+	Constraint *constraints;
+	int numconstraints;
+	int i;
 
-	if (argc % 2 != 0) {
-		DIE("number of arguments must be a multiple of 2 "
-		    "(pairs of search types and queries)\n");
-	}
+	if (argc % 2 != 0)
+		DIE("arguments must be pairs of search types and queries\n");
 
-	constraints = malloc(sizeof(struct _constraints)*argc/2);
+	numconstraints = get_constraints(argc, argv, &constraints);
+	if (numconstraints < 0)
+		return -1;
 
-	for (i = 0; i < argc; i += 2) {
-		type = get_search_type(argv[i]);
-		if (type < 0) {
-			free(constraints);
-			return -1;
-		}
-
-		constraints[numconstraints].type = type;
-		constraints[numconstraints].query = argv[i+1];
-		numconstraints++;
-	}
-
-	mpd_startSearch(conn, 0);
+	mpd_startSearch(conn, exact);
 
 	for (i = 0; i < numconstraints; i++) {
 		mpd_addConstraintSearch(conn, constraints[i].type,
@@ -783,16 +781,71 @@ int cmd_search ( int argc, char ** argv, mpd_Connection * conn )
 	}
 
 	free(constraints);
+
 	mpd_commitSearch(conn);
 	printErrorAndExit(conn);
 
-	while ((entity = mpd_getNextInfoEntity(conn))) {
+	print_filenames(conn);
+
+	my_finishCommand(conn);
+
+	return 0;
+}
+
+int cmd_search ( int argc, char ** argv, mpd_Connection * conn ) 
+{
+	return do_search(argc, argv, conn, 0);
+}
+
+int cmd_find ( int argc, char ** argv, mpd_Connection * conn ) 
+{
+	return do_search(argc, argv, conn, 1);
+}
+
+int cmd_list ( int argc, char ** argv, mpd_Connection * conn ) 
+{
+	Constraint *constraints;
+	int numconstraints = 0;
+	int type;
+	int i;
+	char *tag;
+
+	type = get_search_type(argv[0]);
+	if (type < 0)
+		return -1;
+
+	argc -= 1;
+	argv += 1;
+
+	if (argc > 0) {
+		if (argc % 2 != 0) {
+			DIE("arguments must be a tag type and "
+			    "optional pairs of search types and queries\n");
+		}
+
+		numconstraints = get_constraints(argc, argv, &constraints);
+		if (numconstraints < 0)
+			return -1;
+	}
+
+	mpd_startFieldSearch(conn, type);
+
+	if (argc > 0) {
+		for (i = 0; i < numconstraints; i++) {
+			mpd_addConstraintSearch(conn, constraints[i].type,
+						toUtf8(constraints[i].query));
+		}
+
+		free(constraints);
+	}
+
+	mpd_commitSearch(conn);
+	printErrorAndExit(conn);
+
+	while ((tag = mpd_getNextTag(conn, type))) {
 		printErrorAndExit(conn);
-		if (entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY)
-			printf("%s\n", fromUtf8(entity->info.directory->path));
-		else if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
-			printf("%s\n", fromUtf8(entity->info.song->file));
-		mpd_freeInfoEntity(entity);
+		printf("%s\n", fromUtf8(tag));
+		free(tag);
 	}
 
 	my_finishCommand(conn);
@@ -828,7 +881,8 @@ int cmd_volume ( int argc, char ** argv, mpd_Connection * conn )
 	return 1;
 }
 
-int cmd_pause ( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_pause(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	mpd_sendPauseCommand(conn,1);
 	my_finishCommand(conn);
@@ -906,7 +960,8 @@ int cmd_crossfade ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-int cmd_version ( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_version(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	printf("mpd version: %i.%i.%i\n",conn->version[0],
 			conn->version[1],conn->version[2]);
@@ -977,7 +1032,7 @@ int cmd_tab ( int argc, char ** argv, mpd_Connection * conn )
 {
 	mpd_InfoEntity * entity;
 	char * sp;
-	mpd_Song * song;
+	struct mpd_song *song;
 
 	mpd_sendListallCommand(conn,"");
 	printErrorAndExit(conn);
@@ -1004,7 +1059,7 @@ int cmd_tab ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-char * DHMS(unsigned long t)
+static char * DHMS(unsigned long t)
 {
 	static char buf[32];	/* Ugh */
 	int days, hours, mins, secs;
@@ -1032,7 +1087,8 @@ char * DHMS(unsigned long t)
 	return buf;
 }
 
-int cmd_stats ( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_stats(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	mpd_Stats *stats;
 	time_t t;
@@ -1058,7 +1114,8 @@ int cmd_stats ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-int cmd_status ( int argc, char ** argv, mpd_Connection * conn )
+int
+cmd_status(mpd_unused  int argc, mpd_unused char **argv, mpd_Connection *conn)
 {
 	print_status(conn);
 	return 0;
