@@ -38,6 +38,8 @@ int cmd_ls ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_lsplaylists ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_load ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_search ( int argc, char ** argv, mpd_Connection * conn ) ;
+int cmd_find ( int argc, char ** argv, mpd_Connection * conn ) ;
+int cmd_list ( int argc, char ** argv, mpd_Connection * conn ) ;
 int cmd_save ( int argc, char ** argv, mpd_Connection * conn ) ;
 int cmd_rm ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_volume ( int argc, char ** argv, mpd_Connection * conn ) ;
