@@ -33,43 +33,75 @@
 #ifndef LIBMPDCLIENT_H
 #define LIBMPDCLIENT_H
 
+#include "song.h"
+
 #ifdef WIN32
 #  define __W32API_USE_DLLIMPORT__ 1
 #endif
 
+#include <stddef.h>
 #include <sys/time.h>
 #include <stdarg.h>
-#define MPD_BUFFER_MAX_LENGTH	50000
-#define MPD_ERRORSTR_MAX_LENGTH	1000
+#ifdef MPD_GLIB
+#include <glib.h>
+#endif
+
 #define MPD_WELCOME_MESSAGE	"OK MPD "
 
-#define MPD_ERROR_TIMEOUT	10 /* timeout trying to talk to mpd */
-#define MPD_ERROR_SYSTEM	11 /* system error */
-#define MPD_ERROR_UNKHOST	12 /* unknown host */
-#define MPD_ERROR_CONNPORT	13 /* problems connecting to port on host */
-#define MPD_ERROR_NOTMPD	14 /* mpd not running on port at host */
-#define MPD_ERROR_NORESPONSE	15 /* no response on attempting to connect */
-#define MPD_ERROR_SENDING	16 /* error sending command */
-#define MPD_ERROR_CONNCLOSED	17 /* connection closed by mpd */
-#define MPD_ERROR_ACK		18 /* ACK returned! */
-#define MPD_ERROR_BUFFEROVERRUN	19 /* Buffer was overrun! */
+enum mpd_error {
+	/** no error */
+	MPD_ERROR_SUCCESS = 0,
+
+	/** timeout trying to talk to mpd */
+	MPD_ERROR_TIMEOUT = 10,
+
+	/** system error */
+	MPD_ERROR_SYSTEM,
+
+	/** unknown host */
+	MPD_ERROR_UNKHOST,
+
+	/** problems connecting to port on host */
+	MPD_ERROR_CONNPORT,
+
+	/** mpd not running on port at host */
+	MPD_ERROR_NOTMPD,
+
+	/** no response on attempting to connect */
+	MPD_ERROR_NORESPONSE,
+
+	/** error sending command */
+	MPD_ERROR_SENDING,
+
+	/** connection closed by mpd */
+	MPD_ERROR_CONNCLOSED,
+
+	/** ACK returned! */
+	MPD_ERROR_ACK,
+
+	/** Buffer was overrun! */
+	MPD_ERROR_BUFFEROVERRUN,
+};
 
-#define MPD_ACK_ERROR_UNK	-1
 #define MPD_ERROR_AT_UNK	-1
 
-#define MPD_ACK_ERROR_NOT_LIST			1
-#define MPD_ACK_ERROR_ARG			2
-#define MPD_ACK_ERROR_PASSWORD			3
-#define MPD_ACK_ERROR_PERMISSION		4
-#define MPD_ACK_ERROR_UNKNOWN_CMD		5
+enum mpd_ack {
+	MPD_ACK_ERROR_UNK = -1,
 
-#define MPD_ACK_ERROR_NO_EXIST			50
-#define MPD_ACK_ERROR_PLAYLIST_MAX		51
-#define MPD_ACK_ERROR_SYSTEM			52
-#define MPD_ACK_ERROR_PLAYLIST_LOAD		53
-#define MPD_ACK_ERROR_UPDATE_ALREADY		54
-#define MPD_ACK_ERROR_PLAYER_SYNC		55
-#define MPD_ACK_ERROR_EXIST			56
+	MPD_ACK_ERROR_NOT_LIST = 1,
+	MPD_ACK_ERROR_ARG = 2,
+	MPD_ACK_ERROR_PASSWORD = 3,
+	MPD_ACK_ERROR_PERMISSION = 4,
+	MPD_ACK_ERROR_UNKNOWN_CMD = 5,
+
+	MPD_ACK_ERROR_NO_EXIST = 50,
+	MPD_ACK_ERROR_PLAYLIST_MAX = 51,
+	MPD_ACK_ERROR_SYSTEM = 52,
+	MPD_ACK_ERROR_PLAYLIST_LOAD = 53,
+	MPD_ACK_ERROR_UPDATE_ALREADY = 54,
+	MPD_ACK_ERROR_PLAYER_SYNC = 55,
+	MPD_ACK_ERROR_EXIST = 56,
+};
 
 #ifdef __cplusplus
 extern "C" {
@@ -93,7 +125,7 @@ typedef enum mpd_TagItems
 	MPD_TAG_NUM_OF_ITEM_TYPES
 } mpd_TagItems;
 
-extern char * mpdTagItemKeys[MPD_TAG_NUM_OF_ITEM_TYPES];
+extern const char *const mpdTagItemKeys[MPD_TAG_NUM_OF_ITEM_TYPES];
 
 /* internal stuff don't touch this struct */
 typedef struct _mpd_ReturnElement {
@@ -101,6 +133,30 @@ typedef struct _mpd_ReturnElement {
 	char * value;
 } mpd_ReturnElement;
 
+enum {
+	/** song database has been updated*/
+	IDLE_DATABASE = 0x1,
+
+	/** a stored playlist has been modified, created, deleted or
+	    renamed */
+	IDLE_STORED_PLAYLIST = 0x2,
+
+	/** the current playlist has been modified */
+	IDLE_PLAYLIST = 0x4,
+
+	/** the player state has changed: play, stop, pause, seek, ... */
+	IDLE_PLAYER = 0x8,
+
+	/** the volume has been modified */
+	IDLE_MIXER = 0x10,
+
+	/** an audio output device has been enabled or disabled */
+	IDLE_OUTPUT = 0x20,
+
+	/** options have changed: crossfade, random, repeat, ... */
+	IDLE_OPTIONS = 0x40,
+};
+
 /* mpd_Connection
  * holds info about connection to mpd
  * use error, and errorStr to detect errors
@@ -109,16 +165,16 @@ typedef struct _mpd_Connection {
 	/* use this to check the version of mpd */
 	int version[3];
 	/* IMPORTANT, you want to get the error messages from here */
-	char errorStr[MPD_ERRORSTR_MAX_LENGTH+1];
-	int errorCode;
+	char errorStr[512];
+	enum mpd_ack errorCode;
 	int errorAt;
 	/* this will be set to MPD_ERROR_* if there is an error, 0 if not */
-	int error;
+	enum mpd_error error;
 	/* DON'T TOUCH any of the rest of this stuff */
 	int sock;
-	char buffer[MPD_BUFFER_MAX_LENGTH+1];
-	int buflen;
-	int bufstart;
+	char buffer[16384];
+	size_t buflen;
+	size_t bufstart;
 	int doneProcessing;
 	int listOks;
 	int doneListOk;
@@ -126,8 +182,18 @@ typedef struct _mpd_Connection {
 	mpd_ReturnElement * returnElement;
 	struct timeval timeout;
 	char *request;
+	int idle;
+	void (*notify_cb) (struct _mpd_Connection *connection, unsigned flags, void *userdata);
+	void (*startIdle) (struct _mpd_Connection *connection);
+	void (*stopIdle) (struct _mpd_Connection *connection);
+	void *userdata;
+#ifdef MPD_GLIB
+        int source_id;
+#endif
 } mpd_Connection;
 
+typedef void (*mpd_NotificationCb) (mpd_Connection *connection, unsigned flags, void *userdata);
+
 /* mpd_newConnection
  * use this to open a new connection
  * you should use mpd_closeConnection, when your done with the connection,
@@ -227,79 +293,20 @@ typedef struct _mpd_Stats {
 	unsigned long dbPlayTime;
 } mpd_Stats;
 
+typedef struct _mpd_SearchStats {
+	int numberOfSongs;
+	unsigned long playTime;
+} mpd_SearchStats;
+
 void mpd_sendStatsCommand(mpd_Connection * connection);
 
 mpd_Stats * mpd_getStats(mpd_Connection * connection);
 
 void mpd_freeStats(mpd_Stats * stats);
 
-/* SONG STUFF */
+mpd_SearchStats * mpd_getSearchStats(mpd_Connection * connection);
 
-#define MPD_SONG_NO_TIME	-1
-#define MPD_SONG_NO_NUM		-1
-#define MPD_SONG_NO_ID		-1
-
-/* mpd_Song
- * for storing song info returned by mpd
- */
-typedef struct _mpd_Song {
-	/* filename of song */
-	char * file;
-	/* artist, maybe NULL if there is no tag */
-	char * artist;
-	/* title, maybe NULL if there is no tag */
-	char * title;
-	/* album, maybe NULL if there is no tag */
-	char * album;
-	/* track, maybe NULL if there is no tag */
-	char * track;
-	/* name, maybe NULL if there is no tag; it's the name of the current
-	 * song, f.e. the icyName of the stream */
-	char * name;
-	/* date */
-	char *date;
-
-	/* added by qball */
-	/* Genre */
-	char *genre;
-	/* Composer */
-	char *composer;
-	/* Performer */
-	char *performer;
-	/* Disc */
-	char *disc;
-	/* Comment */
-	char *comment;
-
-	/* length of song in seconds, check that it is not MPD_SONG_NO_TIME  */
-	int time;
-	/* if plchanges/playlistinfo/playlistid used, is the position of the
-	 * song in the playlist */
-	int pos;
-	/* song id for a song in the playlist */
-	int id;
-} mpd_Song;
-
-/* mpd_newSong
- * use to allocate memory for a new mpd_Song
- * file, artist, etc all initialized to NULL
- * if your going to assign values to file, artist, etc
- * be sure to malloc or strdup the memory
- * use mpd_freeSong to free the memory for the mpd_Song, it will also
- * free memory for file, artist, etc, so don't do it yourself
- */
-mpd_Song * mpd_newSong(void);
-
-/* mpd_freeSong
- * use to free memory allocated by mpd_newSong
- * also it will free memory pointed to by file, artist, etc, so be careful
- */
-void mpd_freeSong(mpd_Song * song);
-
-/* mpd_songDup
- * works like strDup, but for a mpd_Song
- */
-mpd_Song * mpd_songDup(mpd_Song * song);
+void mpd_freeSearchStats(mpd_SearchStats * stats);
 
 /* DIRECTORY STUFF */
 
@@ -325,7 +332,7 @@ void mpd_freeDirectory(mpd_Directory * directory);
 /* mpd_directoryDup
  * works like strdup, but for mpd_Directory
  */
-mpd_Directory * mpd_directoryDup(mpd_Directory * directory);
+mpd_Directory * mpd_directoryDup(const mpd_Directory * directory);
 
 /* PLAYLISTFILE STUFF */
 
@@ -351,7 +358,7 @@ void mpd_freePlaylistFile(mpd_PlaylistFile * playlist);
 /* mpd_playlistFileDup
  * works like strdup, but for mpd_PlaylistFile
  */
-mpd_PlaylistFile * mpd_playlistFileDup(mpd_PlaylistFile * playlist);
+mpd_PlaylistFile * mpd_playlistFileDup(const mpd_PlaylistFile * playlist);
 
 /* INFO ENTITY STUFF */
 
@@ -373,7 +380,7 @@ typedef struct mpd_InfoEntity {
 	/* the actual data you want, mpd_Song, mpd_Directory, etc */
 	union {
 		mpd_Directory * directory;
-		mpd_Song * song;
+		struct mpd_song *song;
 		mpd_PlaylistFile * playlistFile;
 	} info;
 } mpd_InfoEntity;
@@ -462,6 +469,9 @@ void mpd_sendLoadCommand(mpd_Connection * connection, const char * name);
 
 void mpd_sendRmCommand(mpd_Connection * connection, const char * name);
 
+void mpd_sendRenameCommand(mpd_Connection *connection, const char *from,
+                           const char *to);
+
 void mpd_sendShuffleCommand(mpd_Connection * connection);
 
 void mpd_sendClearCommand(mpd_Connection * connection);
@@ -504,7 +514,7 @@ void mpd_sendVolumeCommand(mpd_Connection * connection, int volumeChange);
 
 void mpd_sendCrossfadeCommand(mpd_Connection * connection, int seconds);
 
-void mpd_sendUpdateCommand(mpd_Connection * connection, char * path);
+void mpd_sendUpdateCommand(mpd_Connection * connection, const char *path);
 
 /* returns the update job id, call this after a update command*/
 int mpd_getUpdateId(mpd_Connection * connection);
@@ -567,6 +577,14 @@ void mpd_sendNotCommandsCommand(mpd_Connection * connection);
  */
 char *mpd_getNextCommand(mpd_Connection *connection);
 
+void mpd_sendUrlHandlersCommand(mpd_Connection * connection);
+
+char *mpd_getNextHandler(mpd_Connection * connection);
+
+void mpd_sendTagTypesCommand(mpd_Connection * connection);
+
+char *mpd_getNextTagType(mpd_Connection * connection);
+
 /**
  * @param connection a MpdConnection
  * @param path	the path to the playlist.
@@ -628,6 +646,30 @@ void mpd_commitSearch(mpd_Connection *connection);
  * mpd_getNextTag to get the results
  */
 void mpd_startFieldSearch(mpd_Connection *connection, int type);
+
+void mpd_startPlaylistSearch(mpd_Connection *connection, int exact);
+
+void mpd_startStatsSearch(mpd_Connection *connection);
+
+void mpd_sendPlaylistClearCommand(mpd_Connection *connection, char *path);
+
+void mpd_sendPlaylistAddCommand(mpd_Connection *connection,
+                                char *playlist, char *path);
+
+void mpd_sendPlaylistMoveCommand(mpd_Connection *connection,
+                                 char *playlist, int from, int to);
+
+void mpd_sendPlaylistDeleteCommand(mpd_Connection *connection,
+                                   char *playlist, int pos);
+
+void mpd_startIdle(mpd_Connection *connection, mpd_NotificationCb notify_cb, void *userdata);
+
+void mpd_stopIdle(mpd_Connection *connection);
+
+#ifdef MPD_GLIB
+void mpd_glibInit(mpd_Connection *connection);
+#endif
+
 #ifdef __cplusplus
 }
 #endif
