@@ -46,7 +46,9 @@ struct _mpc_table {
 	                       */
 	cmdhandler handler;
 	const char *usage;
-	char *help;           /* NULL means they won't be shown in help */
+
+	/** NULL means they won't be shown in help */
+	const char *help;
 } mpc_table [] = {
 	/* command,     min, max, pipe, handler,         usage, help */
 	{"add",         0,   -1,  1,    cmd_add,         "<file>", "Add a song to the current playlist"},
@@ -76,6 +78,8 @@ struct _mpc_table {
 	{"repeat",      0,   1,   0,    cmd_repeat,      "<on|off>", "Toggle repeat mode, or specify state"},
 	{"random",      0,   1,   0,    cmd_random,      "<on|off>", "Toggle random mode, or specify state"},
 	{"search",      2,   -1,  0,    cmd_search,      "<type> <query>", "Search for a song"},
+	{"find",        2,   -1,  0,    cmd_find,        "<type> <query>", "Find a song (exact match)"},
+	{"list",        1,   -1,  0,    cmd_list,        "<type> [<type> <query>]", "Show all tags of <type>"},
 	{"crossfade",   0,   1,   0,    cmd_crossfade,   "[<seconds>]", "Set and display crossfade settings"},
 	{"update",      0,   -1,  2,    cmd_update,      "[<path>]", "Scans music directory for updates"},
 	{"stats",       0,   -1,  0,    cmd_stats,       "", "Displays statistics about MPD"},
@@ -87,7 +91,7 @@ struct _mpc_table {
 	/* status was added for pedantic reasons */
 	{"status",      0,   -1,  0,    cmd_status,      "", NULL},
 	/* don't remove this, when mpc_table[i].command is NULL it will terminate the loop */
-	{NULL}
+	{ .command = NULL }
 };
 
 static int print_help(char * progname, char * command)
@@ -136,8 +140,8 @@ static int print_help(char * progname, char * command)
 
 static mpd_Connection * setup_connection (void)
 {
-	char * host = DEFAULT_HOST;
-	char * port = DEFAULT_PORT;
+	const char * host = DEFAULT_HOST;
+	const char * port = DEFAULT_PORT;
 	int iport;
 	char * test;
 	int port_env = 0;
