@@ -35,6 +35,7 @@ void printErrorAndExit(mpd_Connection * conn)
 {
 	if(conn->error) {
 		fprintf(stderr,"error: %s\n",fromUtf8(conn->errorStr));
+		mpd_closeConnection(conn);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -77,7 +78,7 @@ void free_pipe_array (int max, char ** array)
 int get_boolean (const char * arg)
 {
 	int i;
-	struct _bool_table {
+	static const struct _bool_table {
 		const char * on;
 		const char * off;
 	} bool_table [] = {
@@ -85,7 +86,7 @@ int get_boolean (const char * arg)
 		{ "1", "0" },
 		{ "true", "false" },
 		{ "yes", "no" },
-		{ NULL }
+		{ .on = NULL }
 	};
 
 	for (i=0; bool_table[i].on; ++i) {
@@ -125,6 +126,31 @@ int get_search_type(const char * arg)
 	return -1;
 }
 
+int get_constraints(int argc, char **argv, Constraint **constraints)
+{
+	int numconstraints = 0;
+	int type;
+	int i;
+
+	assert(argc > 0 && argc % 2 == 0);
+
+	*constraints = malloc(sizeof(Constraint)*argc/2);
+
+	for (i = 0; i < argc; i += 2) {
+		type = get_search_type(argv[i]);
+		if (type < 0) {
+			free(*constraints);
+			return -1;
+		}
+
+		(*constraints)[numconstraints].type = type;
+		(*constraints)[numconstraints].query = argv[i+1];
+		numconstraints++;
+	}
+
+	return numconstraints;
+}
+
 /* note - return value is success; the parsed int itself is in ret */
 
 int parse_int(const char * str, int * ret)
@@ -188,7 +214,7 @@ int parse_int_value_change(const char * str, struct int_value_change * ret)
         return 1;
 }
 
-char * appendToString(char * dest, const char * src, int len) {
+static char * appendToString(char * dest, const char * src, int len) {
 	int destlen;
 
 	if(dest == NULL) {
@@ -207,7 +233,7 @@ char * appendToString(char * dest, const char * src, int len) {
 	return dest;
 }
 
-char * skipFormatting(char * p) {
+static const char * skipFormatting(const char * p) {
 	int stack = 0;
 		
 	while (*p != '\0') {
@@ -231,17 +257,19 @@ char * skipFormatting(char * p) {
 }
 
 /* this is a little ugly... */
-char * songToFormatedString (mpd_Song * song, const char * format, char ** last)
+static char *
+songToFormatedString(struct mpd_song *song,
+		     const char *format, const char ** last)
 {
 	char * ret = NULL;
-	char *p, *end;
+	const char *p, *end;
 	char * temp;
 	int length;
 	int found = 0;
 	int labelFound = 0;
 
 	/* we won't mess up format, we promise... */
-	for (p = (char *)format; *p != '\0'; )
+	for (p = format; *p != '\0'; )
 	{
 		if (p[0] == '|') {
 			++p;
@@ -336,38 +364,50 @@ char * songToFormatedString (mpd_Song * song, const char * format, char ** last)
 		length = end - p + 1;
 
 		labelFound = 0;
-		
-		if(*end != '%')
+
+		if (*end != '%') {
 			length--;
-		else if (strncmp("%file%", p, length) == 0) {
+		} else if (strncmp("%file%", p, length) == 0) {
 			temp = fromUtf8(song->file);
-		}
-		else if (strncmp("%artist%", p, length) == 0) {
+		} else if (strncmp("%artist%", p, length) == 0) {
 			labelFound = 1;
 			temp = song->artist ? fromUtf8(song->artist) : NULL;
-		}
-		else if (strncmp("%title%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->title ? fromUtf8(song->title) : NULL;
-		}
-		else if (strncmp("%album%", p, length) == 0) {
+		} else if (strncmp("%album%", p, length) == 0) {
 			labelFound = 1;
 			temp = song->album ? fromUtf8(song->album) : NULL;
-		}
-		else if (strncmp("%track%", p, length) == 0) {
+		} else if (strncmp("%track%", p, length) == 0) {
 			labelFound = 1;
 			temp = song->track ? fromUtf8(song->track) : NULL;
-		}
-		else if (strncmp("%name%", p, length) == 0) {
+		} else if (strncmp("%title%", p, length) == 0) {
+			labelFound = 1;
+			temp = song->title ? fromUtf8(song->title) : NULL;
+		} else if (strncmp("%name%", p, length) == 0) {
 			labelFound = 1;
 			temp = song->name ? fromUtf8(song->name) : NULL;
-		}
-		else if (strncmp("%time%", p, length) == 0) {
+		} else if (strncmp("%date%", p, length) == 0) {
+			labelFound = 1;
+			temp = song->date ? fromUtf8(song->date) : NULL;
+		} else if (strncmp("%genre%", p, length) == 0) {
+			labelFound = 1;
+			temp = song->genre ? fromUtf8(song->genre) : NULL;
+		} else if (strncmp("%composer%", p, length) == 0) {
+			labelFound = 1;
+			temp = song->composer ? fromUtf8(song->composer) : NULL;
+		} else if (strncmp("%performer%", p, length) == 0) {
+			labelFound = 1;
+			temp = song->performer ? fromUtf8(song->performer) : NULL;
+		} else if (strncmp("%disc%", p, length) == 0) {
+			labelFound = 1;
+			temp = song->disc ? fromUtf8(song->disc) : NULL;
+		} else if (strncmp("%comment%", p, length) == 0) {
+			labelFound = 1;
+			temp = song->comment ? fromUtf8(song->comment) : NULL;
+		} else if (strncmp("%time%", p, length) == 0) {
 			labelFound = 1;
 			if (song->time != MPD_SONG_NO_TIME) {
 				char s[10];
 				snprintf(s, 9, "%d:%02d", song->time / 60, 
-						song->time % 60);
+				                          song->time % 60);
 				/* nasty hack to use static buffer */
 				temp = fromUtf8(s);
 			}
@@ -389,7 +429,7 @@ char * songToFormatedString (mpd_Song * song, const char * format, char ** last)
 	return ret;
 }
 
-void print_formatted_song (mpd_Song * song, const char * format)
+static void print_formatted_song(struct mpd_song *song, const char * format)
 {
 	char * str = songToFormatedString(song, format, NULL);
 
@@ -401,7 +441,7 @@ void print_formatted_song (mpd_Song * song, const char * format)
 
 #define DEFAULT_FORMAT "[%name%: &[%artist% - ]%title%]|%name%|[%artist% - ]%title%|%file%"
 
-void pretty_print_song (mpd_Song * song)
+void pretty_print_song(struct mpd_song *song)
 {
 	/* was a format string specified? */
 	if (get_option("format")->set)
@@ -414,3 +454,15 @@ void pretty_print_song (mpd_Song * song)
 		print_formatted_song(song, DEFAULT_FORMAT);
 	}
 }
+
+void print_filenames(mpd_Connection *conn)
+{
+	mpd_InfoEntity *entity;
+
+	while ((entity = mpd_getNextInfoEntity(conn))) {
+		printErrorAndExit(conn);
+		if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
+			printf("%s\n", fromUtf8(entity->info.song->file));
+		mpd_freeInfoEntity(entity);
+	}
+}
