@@ -30,14 +30,21 @@ struct type##_value_change { \
 
 VALUE_CHANGE(int) /* struct int_value_change */
 
+typedef struct _Constraint {
+	int type;
+	char *query;
+} Constraint;
+
 void printErrorAndExit(mpd_Connection * conn);
 void free_pipe_array (int max, char ** array);
 int stdinToArgArray(char *** array);
 int get_boolean (const char * arg);
 int get_search_type(const char * arg);
+int get_constraints(int argc, char **argv, Constraint **constraints);
 int parse_int(const char *, int *);
 int parse_songnum(const char *, int *);
 int parse_int_value_change(const char *, struct int_value_change *);
-void pretty_print_song (mpd_Song * song);
+void pretty_print_song(struct mpd_song *song);
+void print_filenames(mpd_Connection *conn);
 
 #endif /* MPC_UTIL_H */	
